;====================================================
; DESCRIPTION: 1. load setup.asm at 0x9000
;              2. load kernel at 0x9200
; FILE NAME: bootsect.asm
;====================================================


[org 0x0]

SETUP_SECT	equ 2	
KERNEL_SECT	equ KERNEL_SIZE	

BOOT_SEG	equ 0x7c0
SETUP_SEG	equ 0x900
KERNEL_SEG	equ 0x920

SETUP_ADDR	equ 0x9000


	jmp BOOT_SEG:start

boot_drv		db 0	; floppy drive A:
sect_per_track	dw 18	; sectors per one track
head_per_drv	dw 2	; floppy has 2 heads
current_sect	dw 1	; current reading sector # (relative)
read_sect		db 0	; it read how many sectors

; define routines for A20 line activation
%include "a20.inc"

start:
	mov ax, cs
	mov ds, ax		; ds=es=0x7c0
	mov es, ax

	mov ss, ax		; stack: 0x7c0:0 -> low memory
	mov sp, 0x0

	mov [boot_drv], dl	; bios store drive number in dl


	mov ax, SETUP_SEG	; read 512bytes of setup.asm
	mov es, ax
	mov si, 0			; store image at es:si
	mov cx, SETUP_SECT	; how many sectors?
	call read_sectors

	mov ax, KERNEL_SEG
	mov es, ax
	mov si, 0
	mov cx, KERNEL_SECT
	call read_sectors

	call a20_try_loop	; activate A20 line

	;-------------------------------------------------------------------------
	; PIC ÃÊ±âÈ­
	;--------------------------------------------------------------------------
	
	; 0x20¹ø Æ÷Æ®´Â ¸¶½ºÅÍ PICÀÇ ÃÊ±âÈ­
	; 0x21¹ø Æ÷Æ®´Â ¸¶½ºÅÍ PIC¿¡ ¸í·É Àü´Þ
	; 0xa0¹ø Æ÷Æ®´Â ½½·¹ÀÌºê PICÀÇ ÃÊ±âÈ­
	; 0xa1¹ø Æ÷Æ®´Â ½½·¹ÀÌºê PIC¿¡ ¸í·É Àü´Þ
	
	; PICÀÇ ¼³Á¤ ¼ø¼­
	; ÃÊ±âÈ­ -> ÀÎÅÍ·´Æ® ¹øÈ£ ÇÒ´ç -> 
	
	; ICW1 = 0x11 
	; ICW1[3] = 0 ¿§Áö Æ®¸®°Å¿¡¼­ ÀÎÅÍ·´Æ® ¹ß»ý
	; ICW1[1] = 0 ¸¶½ºÅÍ/½½·¹ÀÌºê ¿¬°áÇØ¼­ »ç¿ë
	; ICW1[0] = 0 ICW4 ¸í·É±îÁö Àü¼Û
	mov al, 0x11
	out 0x20, al				; 20¹ø Æ÷Æ®¿¡ 0x11 ¾²±â

	dw 0x00eb						; PIC°¡ ¼³Á¤µÇ´Â ½Ã°£À» ±â´Ù¸®±â À§ÇÑ µô·¹ÀÌ, ´ÙÀ½ ÁÙ·Î ³Ñ¾î°¡±â
	dw 0x00eb						
	
	out 0xa0, al				; 0xa0¹ø Æ÷Æ®¿¡ 0x11 ¾²±â
	dw 0x00eb, 0x00eb	
	
	; ICW2 = 0x20 0¹ø ÇÉ¿¡ ÀÎÅÍ·´Æ®°¡ ¹ß»ýÇÏ¸é CPU¿¡°Ô 20¹ø ÀÎÅÍ·´Æ®¶ó°í ¾Ë¸²
	; ¸¶½ºÅÍ PICÀÇ Ã¹¹øÂ° ÀÎÅÍ·´Æ®´Â IRQ20 ÀÌ µÈ´Ù
	mov al, 0x20
	out 0x21, al				; 0x21 <- 0x20
	dw 0x00eb, 0x00eb
	
	; ICW2 = 0x28 ½½·¹ÀÌºê PICÀÇ Ã¹¹øÂ° ÀÎÅÍ·´Æ®´Â IRQ28 ÀÌ´Ù
	mov al, 0x28
	out 0xa1, al				; 0xa1 <- 0x28
	dw 0x00eb, 0x00eb
	
	; ICW3Àº ¸¶½ºÅÍ¿Í ½½·¹ÀÌºê°¡ ´Ù¸¥ ÀÇ¹Ì¸¦ °¡Áø´Ù.

	; ICW3 = 0x04 2¹ø ÇÉ¿¡ ½½·¹ÀÌºê PIC°¡ ¿¬°áµÇ¾î ÀÖ´Ù IRQ2´Â ½½·¹ÀÌºê PIC¿¡¼­ Àü¼ÛµÈ ÀÎÅÍ·´Æ®ÀÌ´Ù
	mov al, 0x04
	out 0x21, al				; 0x21 <- 0x04
	dw 0x00eb, 0x00eb
	
	; ICW3 = 0x02 ½½·¹ÀÌºê PIC¿¡°Ô ¸¶½ºÅÍ PICÀÇ 2¹ø ÇÉ¿¡ ¿¬°áµÈ »óÅÂ¶ó´Â °ÍÀ» ¾Ë·ÁÁØ´Ù
	mov al, 0x02
	out 0xa1, al				; 0xa1 <- 0x02
	dw 0x00eb, 0x00eb
	
	; ICW4 = 0000 0001
	; ICW4[1] = 0 PIC°¡ CPU¿¡°Ô ÀÎÅÍ·´Æ®¸¦ ¾Ë¸° ÀÎÅÍ·´Æ® ÇÚµé·¯¿¡¼­ PIC¸¦ ÃÊ±âÈ­ÇØÁØ´Ù 
	;             ±×·¡¾ß ´ÙÀ½ ÀÎÅÍ·´Æ®¸¦ ¹ÞÀ» ¼ö ÀÖ´Ù
	; ICW4[0] = 1 8086 °è¿­ ÇÁ·Î¼¼¼­¿¡ ¿¬°áÇØ¼­ »ç¿ëµÈ´Ù
	mov al, 0x01
	out 0x21, al				; 0x12 <- 0x01
	dw 0x00eb, 0x00eb

	out 0xa1, al				; 0xa1 <- 0x01
	dw 0x00eb, 0x00eb
	
	; ¾ÆÁ÷ ¿ÜºÎ ÀÎÅÍ·´Æ®¸¦ ¹ÞÀ» ÁØºñ°¡ ¾ÈµÇ¾ú±â ¶§¹®¿¡ ÀÎÅÍ·´Æ®¸¦ ¸·´Â´Ù
	
	; ºñÆ®°¡ 1ÀÌ¸é ÇØ´ç ÇÉÀÇ ÀÎÅÍ·´Æ® ¸¶½ºÆ®
	mov al, 0xff				; ½½·¹ÀÌºê PICÀÇ ¸ðµç ÀÎÅÍ·´Æ®¸¦ ¸¶½ºÅ©
	out 0xa1, al				; 0xa1 <- 0xff
	dw 0x00eb, 0x00eb
	
	; 3¹øÂ° ºñÆ®°¡ 0ÀÌ¹Ç·Î ½½·¹ÀÌºê PIC°¡ ¿¬°áµÈ 2¹ø ÇÉ¸¸ ÀÎÅÍ·´Æ® Çã¿ë
	mov al, 0xfb				; 0x21 <- 0xfb = 1111 1011 (3¹øÂ° ºñÆ® = IRQ2)
	out 0x21, al				; ¸¶½ºÅÍ PICÀÇ IRQ2¸¦ Á¦¿ÜÇÑ ³ª¸ÓÁö ÀÎÅÍ·´Æ® ¸¶½ºÅ©
	

	mov ax, 0xb800
	mov es, ax
	mov bx, 0x0
	mov al, byte [hello_msg]	; Greeting!
	mov byte [es:bx], al
	
	
	mov ax, KERNEL_SECT

	;==========================================
	; segment addres = 0x0
	; offset = 0x9000
	; Segment must be 0x0000.
	;==========================================
	jmp 0x0:SETUP_ADDR	; execute setup

	jmp $				; or die!


; es: segment address
; cx: sector count
read_sectors:
	push cx
	call read_one_sect		; load one sector
	pop cx

	add si, 512				; offset for next sector
	dec cx					; next sector
	
	or cx, cx
	jz _read_sectors

	jmp read_sectors
_read_sectors:
	ret



read_one_sect:
	; relative sector # (ax) / sectors per one track (di)
	; = relative track # (ax), absolute sector # in track (dx)
	mov ax, [current_sect]
	mov di, [sect_per_track]
	xor dx, dx
	div di

	; sector # is 1-based count
	inc dl
	mov byte [read_sect], dl

	; relative track # (ax) / headers count
	; = absolute track # (ax), head number (dx)
	xor dx, dx
	div word [head_per_drv]

	mov ch, al			; track(cylinder) #
	mov cl, [read_sect]	; sector #
	mov ah, 2			; service #
	mov al, 1			; sector count
	mov dh, dl			; head #
	mov dl, [boot_drv]	; floppy
	mov bx, si			; offset
	int 0x13

	; next sector to be read
	inc byte [current_sect]

	ret


hello_msg	db "C"

times 510-($-$$) db 0
	dw 0xaa55



