%include "init.inc"


PAGE_DIR	equ 0x2000
PAGE_TBL_TABLE	equ 0x3000
PAGE_TBL_USER	equ 0x4000
PAGE_TBL_KERNEL	equ 0x5000

[bits 16]
[org 0x9000]

setup_start:

	mov word [kernel_size], ax

	mov ax, cs		; cs=ds=es=0x0
	mov ds, ax
	mov es, ax

	mov ax, 0xb800
	mov es, ax
	mov bx, 0x2
	mov al, byte [setup_msg]
	mov byte [es:bx], al
	
	xor eax, eax
	lea eax, [tss]
	mov [descriptor4+2], ax
	shr eax, 16
	mov [descriptor4+4], al
	mov [descriptor4+7], ah


	lea si, [gdt]	; source : ds:si=0x0:gdt
	xor ax, ax
	mov es, ax		; destination : es:di=0x0:0x1000
	mov di, 0x1000
	mov cx, 8*9	; 7 descriptors

	rep movsb

	cli
	lgdt [gdtr]


	mov eax, cr0
	or eax, 0x0000001
	mov cr0, eax

	jmp $+2
	nop
	nop

	jmp dword KERNEL_CODE_SEL:protect_start


[bits 32]
protect_start:
	mov ax, 0x10
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax
	mov ss, ax

	mov edi, 0xb8004
	mov al, byte [protect_msg]
	mov byte [edi], al

	mov esp, 0x9ffff


	;
	; kernel exists at 0x100000~
	;
	mov ax, [kernel_size]
	mov dx, 512
	mul dx
	mov cx, ax

	mov esi, 0x9200
	mov edi, 0x100000
	
copy_kernel:
	mov al, byte [ds:esi]
	mov byte [es:edi], al
	inc esi
	inc edi
	dec cx
	jnz copy_kernel	

	
	; clear PAGE_DIR ~ PAGE_TBL_KERNEL
	mov edi, PAGE_DIR
	xor eax, eax
	mov ecx, 4096		; 4096*4=16K=4pages
	cld
	rep stosd

	; set PAGE DIRECTORY

	mov edi, PAGE_DIR+0x0*4		; 0th table = 0x3000
	mov eax, PAGE_TBL_TABLE
	or eax, 0x7
	mov [es:edi], eax

	mov edi, PAGE_DIR+0x100*4	; 0x100th table = 0x4000
	mov eax, PAGE_TBL_USER
	or eax, 0x7
	mov [es:edi], eax

	mov edi, PAGE_DIR+0x300*4	; 0x300th table = 0x5000
	mov eax, PAGE_TBL_KERNEL
	or eax, 0x3
	mov [es:edi], eax

	;
	; set PAGE TABLE for global tables
	; physical memory 0x0~0x100000 is mapped at virtual memory 0x0~0x100000
	;
	mov edi, PAGE_TBL_TABLE
	mov eax, 0x00000007
	mov ecx, 256
set_tbl_table:
	mov [es:edi], eax
	add eax, 0x1000
	add edi, 4
	dec ecx
	jnz set_tbl_table

	;
	; set PAGE TABLE for user space
	; user space is at physical memory 0x200000~0x201000 (2M)
	; user space is mapped at 0x40000000~0x40001000
	mov edi, PAGE_TBL_USER
	mov eax, 0x200000
	or eax, 0x7
	mov [es:edi], eax

	;
	; set PAGE TABLE for kernel space
	; kernel space is at physical memory 0x100000~0x102000
	; kernel space is mapped at 0xc0000000~0xc0002000
	; NOW kernel size must be smaller than 8K
	;
	mov edi, PAGE_TBL_KERNEL+0x4*0
	mov eax, 0x100000
	or eax, 3
	mov [es:edi], eax

	mov edi, PAGE_TBL_KERNEL+0x4*1
	mov eax, 0x101000
	or eax, 3
	mov [es:edi], eax

	mov edi, PAGE_TBL_KERNEL+0x4*2
	mov eax, 0x102000
	or eax, 3
	mov [es:edi], eax

	mov edi, PAGE_TBL_KERNEL+0x4*3
	mov eax, 0x103000
	or eax, 3
	mov [es:edi], eax


	mov eax, PAGE_DIR
	mov cr3, eax

	mov eax, cr0
	or eax, 0x80000000
	mov cr0, eax

	; stack space is grow down from 0x9ffff
	;mov esp, 0xc009ffff
	mov edi, 0xb8006
	mov al, 'S' 
	mov byte [edi], al

	;jmp 0xc0000000

	jmp $

setup_msg	db "a", 0
protect_msg	db "O", 0

kernel_size db 0

gdtr:
	dw 0x1000
	dd 0x1000

gdt:
	dd 0x00000000, 0x00000000
	dd 0x0000FFFF, 0x00CF9A00 ; kernel code segment
	dd 0x0000FFFF, 0x00CF9200 ; kernel data segment
	dd 0x8000FFFF, 0x0040920B ; video memory segment


	;dd 0x00000000, 0x00000000	; for TSS descriptor
descriptor4:
	dw 104
	dw 0
	db 0
	db 0x89
	db 0
	db 0

	dd 0x0000FFFF, 0x00cfFA00  ; user code segment
	dd 0x0000FFFF, 0x00cfF200  ; user data segment
	dd 0x00000000, 0x00000000
gdt_end:

tss: times 104 db 0


times 1024-($-$$) db 0
